#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "grb2.h"
#include "wgrib2.h"
#include "fnlist.h"

/*
 * Write Grib -ieee interface
 *
 * routines to modify grib fields
 *
 * 3/2008 Public Domain by Wesley Ebisuzaki
 *
 */


extern const char *level_table[192];

/*
 * HEADER:100:set_lev:misc:1:changes level code .. not complete
 */

int f_set_lev(ARG1) {
    unsigned char *p;
    float val1;
    int pdt, i, scale;
    char string[100];

    if (mode < 0) return 0;
    pdt = code_table_4_0(sec);
    p = sec[4];

    if (pdt == 0) {
        p[22] = p[23] = p[24] = p[25] = p[26] = p[27] = p[28] = p[29] = 
		p[31] = p[32] = p[33] = 255;

	for (i = 1; i < 192; i++) {
	    if (strcmp("reserved", level_table[i]) == 0) continue;
	    if (strchr(level_table[i],'%') == NULL) {
		/* level type with no parameter */
	        if (strcmp(arg1, level_table[i]) == 0) {
 	            p[22] = i;
 	            return 0;
	        }
	    }
	    else {
		if (sscanf(arg1,level_table[i],&val1) == 1) {
		   sprintf(string,level_table[i], val1);
		   if (strcmp(string, arg1) == 0) {
			if (i == 100 || i == 108) val1 = val1 * 100; // convert mb to Pa
			p[22] = i;
                        scale = 0;
			if ((int) val1 == val1) {
			    p[23] = scale;
			    int_char((int) val1, p+24);
			    return 0;
			}
		   }
		}
	    }
        }
    }
    fatal_error("need to modify set_lev for %s", arg1);
    return 0;
}


/*
 * HEADER:100:set_date:misc:1:changes date code .. not complete .. only analysis/fcst
 */

int f_set_date(ARG1) {

    unsigned char *p;
    int year, month, day, hour, minute, second, i, j, units, dtime;

    if (mode == -1) {
	return 0;
    }
    if (mode < 0) return 0;

        p = arg1;
        day = 1;
        second = minute = hour = 0;

        if (sscanf(p,"%4d",&year) == 0) fatal_error("set_date: year %s", arg1);
	p += 4;
        if (sscanf(p,"%2d",&month) == 0) fatal_error("set_date: month %s", arg1);
	p += 2;
        if (sscanf(p,"%2d",&day) == 0) {
	    p += 2;
            if (sscanf(p,"%2d",&hour) == 0) {
	        p += 2;
                if (sscanf(p,"%2d",&minute) == 0) {
	            p += 2;
                    sscanf(p,"%2d",&second);
		}
	    }
	}

        p = sec[1];

	uint2_char(year, p+12);
	p[14] = month;
	p[15] = day;
	p[16] = hour;
	p[17] = minute;
	p[18] = second;

    i = code_table_4_0(sec);
    j = 0;
    if (i == 8) j = 34;
    else if (i == 9) j = 47;
    else if (i == 10) j = 35;
    else if (i == 11) j = 37;
    else if (i == 12) j = 36;
    else if (i == 13) j = 68;
    else if (i == 14) j = 64;

        if (j == 0) return 0;

        units = code_table_4_4(sec);
        if (units == -1) return 1;

        dtime = pds_fcst_time(sec);
        if (dtime == 0xffffffff) return 0;

        return add_time(&year, &month, &day, &hour, &minute, &second, dtime, units);

	p = sec[4] + j;
	uint2_char(year, p);
	p += 2;
	*p++ = month;
	*p++ = day;
	*p++ = hour;
	*p++ = minute;
	*p++ = second;
    return 0;
}	

extern struct gribtab_s gribtab[];


/*
 * HEADER:100:set_var:misc:1:changes variable name
 */

int f_set_var(ARG1) {
    struct gribtab_s *p;
    if (mode == -1) {
	p = gribtab;
	while (p->disc != -1) {
	    if (strcmp(arg1,p->name) == 0) {
		*local = (void *) p;
		return 0;
	    }
	    p++;
	}
	fatal_error("set_var: could not find %s", arg1);
    }
    else if (mode >= 0) {
	p = *local;
	sec[0][6] = p->disc;
        sec[1][9] = p->mtab;
        sec[1][10] = p->ltab;
        sec[4][9] = p->pcat;
        sec[4][10] = p->pnum;
    }
    return 0;
}

/*
 * HEADER:100:set_center:misc:1:changes center X = C or C:S     C and S are center/subcenter numbers
 */

int f_set_center(ARG1) {
   int i, center, subcenter;

   if (mode >= 0) {
       i = sscanf(arg1,"%d:%d", &center, &subcenter);
       if (i == 0) fatal_error("set_center: bad arg %s", arg1);
       int2_char(center, sec[1]+5);
       if (i == 2) int2_char(subcenter, sec[1]+7);
    }
    return 0;
}
